library(shiny)

shinyServer(function(input, output,session) {

   critvals <- reactive({
        alpha <- as.numeric(input$alpha)
        qn1 <- ifelse(input$alt=="Not equal to",qnorm(1 - alpha/2),qnorm(1 - alpha))
        qn2 <- ifelse(input$alt=="Not equal to",qnorm(0.975),qnorm(0.95))
        
        if(input$alt=="Less than") z <- (-1)
        if(input$alt=="Greater than") z <- 1
        if(input$alt=="Not equal to") z <- c(-1,1)
        cv1 <- 50 + z*qn1*input$sigma/sqrt(input$n)
        cv2 <- 50 + z*qn2*5/sqrt(50)
        list(cv1=cv1,cv2=cv2,qn1=qn1,qn2=qn2)
   
   }) 

   power <- reactive({
        alpha <- as.numeric(input$alpha)
        if(input$alt=="Less than") x <- seq(50 - 6 * input$sigma/sqrt(input$n),50, length = 200)
        if(input$alt=="Greater than") x <- seq(50, 50 + 6 * input$sigma/sqrt(input$n), length = 200)        
        if(input$alt=="Not equal to") {
            x <- seq(50- 6 * input$sigma/sqrt(input$n), 50 + 6 * input$sigma/sqrt(input$n), length = 200)        
            alpha <- alpha/2
        }    
        qn1 <- critvals()[[3]]
        qn2 <- critvals()[[4]]

        if(input$alt=="Less than") {
            y1 <- 1-pnorm(qn1 - sqrt(input$n) * abs(50 - x)/input$sigma)
            y2 <- 1-pnorm(qn1 - sqrt(50) * abs(50 - x)/5)
        }
        if(input$alt=="Greater than") {
            y1 <- 1-pnorm(qn2 - sqrt(input$n) * abs(50 - x)/input$sigma) 
            y2 <- 1-pnorm(qn2 - sqrt(50) * abs(50 - x)/5)              
        }      
        if(input$alt=="Not equal to") {
            y1 <- 1 - (pnorm(qn1 - sqrt(input$n) * abs(50 - x)/input$sigma) - 
                      pnorm(-qn1 - sqrt(input$n) * abs(50 - x)/input$sigma))
            y2 <- 1 - (pnorm(qn2 - sqrt(50) * abs(50 - x)/5) - 
                      pnorm(-qn2 - sqrt(50) * abs(50 - x)/5))              
        }              
        cbind(x,y1,y2)
        
   })   
   
   
   output$text1 <- renderText({
          if(input$alt=="Less than") txt <- "&lt;"
          if(input$alt=="Greater than") txt <- "&gt;"
          if(input$alt=="Not equal to") txt <- "&ne;"
          line <- paste("<h4>Testing H<sub>0</sub>: &mu;=50
                   vs. H<sub>a</sub>: &mu;",txt  ,50,"</h4>")    
          line[2] <- paste("<h4>Sample Size: ",input$n,"</h4>")
          line[3] <- paste("<h4>Standard Deviation: ",input$sigma,"</h4>")
          line[4] <- paste("<h4>Type I error probability &alpha;: ",as.numeric(input$alpha),"</h4>")
          if(input$alt=="Less than") line[5] <- paste("<h4>Reject H<sub>0</sub> if X&#773; &lt;",
                    round(critvals()[[1]],1),"</h4>")
          if(input$alt=="Greater than") line[5] <- paste("<h4>Reject H<sub>0</sub> if X&#773; &gt;",
                    round(critvals()[[1]],1),"</h4>")
          if(input$alt=="Not equal to") line[5] <- paste("<h4>Reject H<sub>0</sub> if X&#773; &lt;",
                    round(critvals()[[1]][1],1)," or X&#773; &gt;",
                    round(critvals()[[1]][2],1),"</h4>")
          line[6] <- paste("<h4>Population mean &mu;: ",input$mu0,"</h4>")                    
          x <- power()[,1]
          y <- power()[,2]
          i <- seq_along(x)[abs(x-input$mu0)==min(abs(x-input$mu0))]
          y <- round(100*y[i],1)[1] 
          print(y)                  
          line[7] <- paste("<h4>Probability to reject H<sub>0</sub> if true mean is ",input$mu0,": ",y,"%</h4>",sep="")
                
          line

   
   })        
   
    
   output$plot1 <- renderPlot({
        par(mfrow=c(1,2))
        x <- power()[,1]
        y1 <- power()[,2]
        y2 <- power()[,3]
        plot(x,y1, xlab = expression(mu), ylab = "Power", type = "l", lwd=3,
            col = "blue",ylim=c(0,1), main="Power Curve",cex=1.3)
        lines(x, y2, lwd=1, col = "red")
        i <- seq_along(x)[abs(x-input$mu0)==min(abs(x-input$mu0))]
        abline(v=x[i],lwd=2)
        points(x[i],y1[i],col="blue",cex=2.5,pch=20)
                
                
        cv1 <- critvals()[[1]]
        cv2 <- critvals()[[2]]
        x <- seq(50- 6 * input$sigma/sqrt(input$n), 50 + 6 * input$sigma/sqrt(input$n), length = 200)
        y1 <- dnorm(x,50,input$sigma/sqrt(input$n))
        y2 <- dnorm(x,50,5/sqrt(50))
        yr <- range(c(y1,y2))           
        plot(x, y1, xlab = expression(mu), ylab = "", type = "l", lwd=3, 
            col= "blue",ylim=yr, main="Rejection Region",cex=1.3)
        lines(x, y2, lwd=1,col = "red")
            abline(v=cv1, lwd=3,col = "blue")        
            abline(v=cv2, lwd=1,col = "red")
        xr <- range(x)
        i1 <- seq_along(x)[abs(x-cv1[1])==min(abs(x-cv1[1]))]
        i2 <- seq_along(x)[abs(x-cv1[2])==min(abs(x-cv1[2]))]
        if(input$alt=="Less than") arrows(cv1[1],y1[i1],xr[1],y1[i1],lwd=3)    
        if(input$alt=="Greater than") arrows(cv1[1],y1[i1],xr[2],y1[i1],lwd=3)
        if(input$alt=="Not equal to") {
            arrows(cv1[1],y1[i1],xr[1],y1[i1],lwd=3)
            arrows(cv1[2],y1[i2],xr[2],y1[i2],lwd=3)
        }    
                
   })
   
   output$appinfo <- renderText({ 
      HTML("<br>&nbsp;<br>&nbsp;<br>&nbsp;<br>
      <a target=\"_blank\" href=\"http://academic.uprm.edu/wrolke/myapps1.htm#powerMean\"><h4>What to do</h4></a><br>
      <h6>Written by: Dr. Wolfgang Rolke<br>Created: 12/6/2014</h6>")
      
  }) 
  output$explainText <- renderText({ 
        paste("<h5>Explanation: This demo illustrates the concepts of the Power of a test
        <br>and how it depends on the the sample size, the true mean, the type I error probability chosen and on the
        the alternative
        <br>red curves are for comparison case with  n=50, &alpha; = 0.05, &mu;=50, &sigma;=5</h5>")
        
  })

})

